@@ -120,10 +120,10 @@ def add_missing(data, cause_dict, m_min=0, m_max=0.5):
                         state_single = random.choice(states)
                         selected_state.append(state_single)
                         states.remove(state_single)
-                        if data[cause].value_counts()[selected_state].sum() > m * data.shape[0]:
+                        if data[cause].value_counts()[selected_state].sum() > m * data.shape[0] or len(states) == 1:
                             m_h = np.random.uniform(m, m * data.shape[0] / data[cause].isin(selected_state).sum())
-                            m_l = (data.shape[0] * m - data[cause].isin(selected_state).sum() * m_h) / data[cause].isin(
-                                states).sum()
+                            m_l = max((data.shape[0] * m - data[cause].isin(selected_state).sum() * m_h) / data[cause].isin(
+                                states).sum(), 0)
                             break
                     data_missing[var][
                         (np.random.uniform(size=len(data)) < m_h) & (data[cause].isin(selected_state))] = None
@@ -640,28 +640,8 @@ def dag2cpdag(dag):
             return cpdag
 
 
-# statistical test
-def independence_test(data, cols, varnames=None, weights=None, test='g-test', threshold=0.05):
-    '''
-    :param data: the unique datapoints as a 2-d array, each row is a datapoint, assumed unique
-    :param cols: the columns (=variables) for the marginal contingency table. columns must be ordered low to high
-    :param test: the name of test
-    :param threshold: the threshold for G-test
-
-    :returns 1st element: test result
-    :returns 2nd element: test score
-    '''
-    if test == 'g-test':
-        p = g_test(data, cols, weights)
-    elif test == 'z-test':
-        p = z_test(data, varnames, cols)
-    else:
-        raise Exception('Undefined test function.')
-    return p > threshold, p
-
-
 # orient v-structrues
-def orient_v_structure(data, varnames, pc, sepset, test, threshold=0.05):
+def orient_v_structure(data, varnames, pc, sepset, ci_test, threshold=0.05):
     dag = {}
     v_candidate = {}
     for var in pc:
@@ -674,9 +654,8 @@ def orient_v_structure(data, varnames, pc, sepset, test, threshold=0.05):
                 for var2 in pc_copy[pc_copy.index(var1) + 1:]:
                     if var2 in sepset[var1]:
                         if var not in sepset[var1][var2]:
-                            cols = np.array([varnames.index(v) for v in [var1, var2] + list(sepset[var1][var2])])
-                            v_candidate[tuple([var, var1, var2])] = \
-                                independence_test(data, cols, varnames=varnames, test=test, threshold=threshold)[1]
+                            cols = [varnames.index(v) for v in [var1, var2] + list(sepset[var1][var2])]
+                            v_candidate[tuple([var, var1, var2])] = ci_test(data, cols)
     while len(v_candidate):
         pair = min(v_candidate, key=v_candidate.get)
         if (pair[0] not in dag[pair[1]]['par']) & (pair[0] not in dag[pair[2]]['par']):
